/*
 * Esta biblioteca e um software livre, que pode ser redistribuido e/ou 
 * modificado sob os termos da Licença Publica Geral Reduzida GNU, 
 * conforme publicada pela Free Software Foundation, versao 2.1 da licenca.
 *
 * Esta biblioteca e distribuida na experanca de ser util aos seus usuarios, 
 * porem NAO TEM NENHUMA GARANTIA, EXPLICITAS OU IMPLICITAS, COMERCIAIS OU 
 * DE ATENDIMENTO A UMA DETERMINADA FINALIDADE. 
 * Veja a Licenca Publica Geral Reduzida GNU para maiores detalhes. 
 * A licenca se encontra no arquivo lgpl-br.txt 
 */
package org.jboleto.bancos;

import org.adempierelbr.util.TextUtil;
import org.jboleto.Banco;
import org.jboleto.JBoletoBean;

/**
 * 		Classe para gerar o Boleto do Banco Santander
 * 	@author Mario Grigioni
 * 		<li>Versão legada
 * 	@author Ricardo Santana (Kenos, www.kenos.com.br)
 * 	@version $Id: Santander_033.java, v1.0 2016/MM/DD 2:25:24 PM, ralexsander Exp $
 */
public class Santander_033 implements Banco
{
	JBoletoBean boleto;
	
	public String D1 = "0";
	public String D2 = "0";
		
    /**
     * Metdodo responsavel por resgatar o numero do banco, coloque no return o codigo do seu banco
     */
	public String getNumero()
	{
        return "033";
    }	//	getNumero
	
	public String getDvNumero()
	{
		return "7";
	}	//	getDvNumero
    
    /**
     * Método paticular do Banco Santander
     */    
    public String getZero() 
    {
        return "0";
    }	//	getZero
    
    /**
     * 	Dígito verificador do Nosso Número
     */
    public String getDvNossoNumero (String nossoNumero)
    {
    	String campo = new StringBuilder (TextUtil.lPad (nossoNumero, 7)).reverse().toString();
    	//
    	int multiplicador = 2;
		int multiplicacao = 0;
		int soma_campo = 0;

		for (int i = campo.length(); i > 0; i--)
		{
			multiplicacao = Integer.parseInt (campo.substring(i-1,i)) * multiplicador;

			soma_campo = soma_campo + multiplicacao;

			multiplicador++;
			if (multiplicador > 9)
				multiplicador = 2;
		}

		int dac = 11 - (soma_campo%11);

		if ((dac == 0 || dac == 1 || dac > 9))
        	dac = 1;

        return String.valueOf (dac);
    }	//	getDvNossoNumero
    
    /**
	 * 	getModulo11 - Santander
	 *  @param String value
	 *  @param int type
	 * 	@return String dac
	 */
    public String getModulo11(String campoLivre)
    {
    	String campo = campoLivre + D1;
    	int multiplicador = 2;
		int multiplicacao = 0;
		int soma_campo = 0;
		
		for (int i = campo.length(); i > 0; i--) {
			multiplicacao = Integer.parseInt(campo.substring(i-1,i)) * multiplicador;
			
			soma_campo = soma_campo + multiplicacao;
			
			multiplicador++;
			if (multiplicador > 7)
				multiplicador = 2;
		}
		
		int dac = 11 - (soma_campo%11);
		
		if (dac == 11){
			dac = 0;
		}
		else if (dac == 10){
            if (Integer.parseInt(D1) == 9){
            	D1 = "0";
            	dac = Integer.parseInt(getModulo11(campoLivre));
            }
            else{
            	D1 = ((Integer)(Integer.parseInt(D1) + 1)).toString();
            	dac = Integer.parseInt(getModulo11(campoLivre));
            }
		}
		
        return ((Integer)dac).toString();
    }	//	getModulo11
    
    /**
     * Método particular do santander
     */
    public String getDacCampoLivre(String campo)
    {
    	D1 = boleto.getModulo10(campo);
    	D2 = getModulo11(campo);
    	
    	return D1+D2;
    }	//	getDacCampoLivre
    
    private String getCampoLivre()
    {       
        String campo =  boleto.getCodCliente() + boleto.getNossoNumero() + getDvNossoNumero(boleto.getNossoNumero()) + getZero() + "101";
        return "9" + campo;
    }	//	getCampoLivre
    
    /**
     * Classe construtura, recebe como parametro a classe jboletobean
     */
	public Santander_033 (JBoletoBean boleto)
	{
		this.boleto = boleto;		
	}	//	Santander_033
	
    /**
     * Metodo que monta o primeiro campo do codigo de barras 
     * Este campo como os demais e feito a partir do da documentacao do banco
     * A documentacao do banco tem a estrutura de como monta cada campo, depois disso
     * é só concatenar os campos como no exemplo abaixo.
     */
	private String getCampo1()
	{
		String campo = getNumero() + String.valueOf(boleto.getMoeda()) + getCampoLivre().substring(0, 5);
		return campo + boleto.getModulo10(campo);		
	}	//	getCampo1
	
    /**
     *  
     */
	private String getCampo2()
	{
		String campo = getCampoLivre().substring(5, 15);		
		return campo + boleto.getModulo10(campo);
	}	//	getCampo2

    /**
     * 
     */
	private String getCampo3()
	{
		String campo = getCampoLivre().substring(15, 25);		
		return campo + boleto.getModulo10(campo);		
	}	//	getCampo3
	
    /**     
     *      
     */
	private String getCampo4()
	{
		String campo = 	getNumero() + String.valueOf(boleto.getMoeda()) + 
						boleto.getFatorVencimento() + boleto.getValorTitulo() + getCampoLivre();							
		return boleto.getModulo11(campo,9);
	}	//	getCampo4
	
    /**
     * 
     */
	private String getCampo5()
	{
		String campo = boleto.getFatorVencimento() + boleto.getValorTitulo();
		return campo;
	}	//	getCampo5
	
    /**
     * Metodo para monta o desenho do codigo de barras
     * A ordem destes campos tambem variam de banco para banco, entao e so olhar na documentacao do seu banco
     * qual a ordem correta
     */
	public String getCodigoBarras()
	{
		String teste = getNumero() + String.valueOf(boleto.getMoeda()) + getCampo4() + getCampo5() + getCampoLivre();		
		return teste;
	}	//	getCodigoBarras

    /**
     * Metodo que concatena os campo para formar a linha digitavel
     * E necessario tambem olhar a documentacao do banco para saber a ordem correta a linha digitavel
     */
	public String getLinhaDigitavel() 
	{
		return 	getCampo1().substring(0,5) + "." + getCampo1().substring(5) + "  " + 
				getCampo2().substring(0,5) + "." + getCampo2().substring(5) + "  " +
				getCampo3().substring(0,5) + "." + getCampo3().substring(5) + "  " +
				getCampo4() + "  " + getCampo5(); 
	}	//	getLinhaDigitavel

	public String getCarteira()
	{
		return "1";
	}	//	getCarteira
}	//	Santander_033